(* $Id: classifications_base.ml,v 1.1 1999/05/13 12:26:49 gerd Exp $
 * ----------------------------------------------------------------------
 *
 *)

open Common;;

(* Define a catalogue. *)

let scheme =
  Scheme.mk_scheme
    [ "title", Field.T_constrained
	         (Field.T_line,  (* Must be T_line in a catalogue *)
		  (fun f -> if f = Field.null then
		    raise(User_error "Field 'title' must not be empty!")));
      "id",    Field.T_line;  (* Must be T_line in a catalogue *)
    ] 
    [ "title" ]
    [ "id" ]
;;


class classifications =
  object (self)
    inherit File_base.t scheme  (!basedir ^ "/classifications.db") "classifications" as super


    method get_derived_field r n =
      match n with
	"id" ->
	  Field.mk_line(string_of_int(r # id()))
      |	_ ->
	  failwith ("Classifications_base: unknown derived field " ^ n)

    method add_record fields =
      try
        super # add_record fields 
      with
        Table.Not_unique "title" ->
	  raise (User_error "This title has already been entered")


    method change_record id fields =
      try
        super # change_record id fields 
      with
        Table.Not_unique "title" ->
	  raise (User_error "This title has already been entered")

  end
;;


Base.register_table 
    "classifications" 
    (new classifications :> Table.t)
;;

Base.register_catalogue_exd
    "classifications"
    "classifications"
    "id"
    "title"
    true
    []
;;

let selection = Base.get_selection "classifications"
;;

(* ======================================================================
 * History:
 * 
 * $Log: classifications_base.ml,v $
 * Revision 1.1  1999/05/13 12:26:49  gerd
 * 	Initial revision.
 *
 * 
 *)
