#! /bin/sh
# (*
. ./env.sh
exec $bin/ocamlfattop "$0"
*) directory ".";;

(* $Id: list.cgi,v 1.13 2000/04/22 17:01:53 gerd Exp $
 * ----------------------------------------------------------------------
 *
 *)

#directory "../lib";;
#directory "../custom";;
#load "lib.cma";;
#load "custom.cma";;

open Common;;
open Authors_base;;
open Links_base;;
open File_base;;
open Wbuffer;;
open Field;;
open Fragment;;
open Record;;


(* CGI parameters:
 * "type":             either "title" or "author" or "date"
 *)


type sorted_by = Title | Author | Date;;


let print_rec r =

  let title = field r "title" in
  let author = field r "author_name" in
  let coauthor = field r "coauthor_name" in
  let release = field r "release" in
  let subtitle = field r "subtitle" in
  let info = field r "info_link" in
  let download = field r "download_link" in
  let date = field r "modification_date" in

  let author_link = field r "author_link" in
  let coauthor_link = field r "coauthor_link" in

  let reporter = field r "reporter_name" in

  let is_original = field r "reporter_id" = Field.null in

  let indent = "<td>&#160;&#160;&#160;&#160;</td>" in

  let ap = encoded_auto_params() in

  let format =
    [ Html "<tr>";
      (if is_original then
         Html "<td rowspan=3 width=30 valign=top><img src=\"icons/clef.gif\" alt=\"original link\" align=top></td>"
       else
         Html "<td rowspan=3 width=30 valign=top><img src=\"icons/dot.gif\" alt=\"reported link\" align=top></td>");

      Html "<td colspan=2><b>";
      Field ("title", title);
      Html "</b> ";

      Html "<font size=-2>";

      (if release <> null then Html "&nbsp;&nbsp;&nbsp;Release " else Html "");
      Field ("release", release);

      Html "&nbsp;&nbsp;&nbsp;";
      Field ("date", date);

      Html "&nbsp;&nbsp;&nbsp;";
      (if is_original then 
	Html "ORIGINAL LINK" 
      else
	Html "");

      Html "</font>";
 
      Html "</td></tr>\n";
      Html "<tr>";
      Html ("<td>");

      Html "\n";
      Field ("subtitle", subtitle);

      Html "\n - written by ";
      if author=null then
	Html "anonymous"
      else
	Field ("author", author);

      begin
	if coauthor <> null then
          List ("",[ Html "\nand "; 
                     Field ("coauthor", coauthor);
		   ])
	else
          Html ""
      end;

      (if not is_original then
	 List ("", [ 
	   Html "; reported by ";
	   Field ("reporter", reporter);
	   Html ". "
	 ])
       else
	 Html ". ");

      Html ("<a href=\"link_editor_orig.cgi?edit_page=show&id=" ^ string_of_int (r#id()) ^ 
	    "&AUTO=" ^ ap ^ "\">");
      Html "Details</a>\n";
      Field ("info_link", info);
      Html "\n";
      Field ("download_link", download);
      Html "\n";
      Field ("author_link", author_link);
      Html "\n";
      Field ("coauthor_link", coauthor_link);

      Html "</td></tr>\n";
      Html "<tr><td height=8></td></tr>\n";
    ] in

  let config =
    [ "info_link",     Out_hyperlink "Info";
      "download_link", Out_hyperlink "Download";
      "author_link",   Out_hyperlink "Author";
      "coauthor_link", Out_hyperlink "Co-Author";
    ] in
       
  print (html_output [ Config(config, format) ])
;;



let main() =
  let type_param = param "type" in

  set_auto_param "frames" (any_param "frames");

  let split_in_sections t crit disp disp_null lst =
    let rec split last_crit last_disp deferred lst =
      match lst with
	[] -> 
	  if deferred <> [] then [ last_disp, List.rev deferred ] else []
      |	r :: lst' ->
	  let this = r # field crit in
	  if Field.compare_fields t this last_crit = 0 then
	    split last_crit last_disp (r :: deferred) lst'
	  else
	    let this_disp = 
	      if this = null then
		disp_null
	      else
		html_output [ Field("noname", r # field disp) ] in
	    if deferred = [] then
	      split this this_disp [r] lst'
	    else
	      (last_disp, List.rev deferred) ::
	      split this this_disp [r] lst'
    in
    split null disp_null [] lst
  in

  let lst = 
    match type_param with
      "author" ->
	let links = author_title_links () in
	split_in_sections 
	  T_line "author_name" "author_name" "Anonymous" links
    | "title" ->
	[ "", title_author_links ()]
    | "date" ->
	split_in_sections
	  T_date "modification_date" "modification_date" "Unknown" 
	  (date_author_links ())
(*    | "classification" ->
	split_in_sections
	  (T_select (Classifications_base.selection))
	  "classification" "classification" "Unclassified"
	  (classification_author_links())
*)
    | "classification" ->
	classified_links()
    | _ ->
	failwith "This type of sorting is not implemented"
  in
      
  print "<html>\n";
  print ("<head><title>The links, sorted by " ^ type_param ^ " </title></head>\n");
  print "<body bgcolor=white text=black link=black alink=black vlink=\"#99A1C5\">\n";
  top_navigator();
  List.iter 
    (fun (header, items) ->
      if header <> "" then begin
	print "<table cellspacing=0 border=0 bgcolor=\"#B9C1E5\" width=\"100%\">\n";
	print "<tr><td align=left><img src=\"icons/empty.gif\" width=30 height=1>";
	print "<font size=4><b>";
	print header;
	print "</b></font>";
	print "</td></tr>";
	print "</table>\n";
      end;
      print "<table cellspacing=0 border=0>\n";
      List.iter print_rec items;
      print "</table>\n";
    )
    lst;
  bottom_navigator();
  print "</body>\n";
  print "</html>\n";
;;

(*
Logging.loglevels :=  [ Logging.Debug; Logging.Param; Logging.Info; 
  Logging.Warn; Logging.Err ];;
*)

html_header();
do_protected main;;

(* ======================================================================
 * History:
 * 
 * $Log: list.cgi,v $
 * Revision 1.13  2000/04/22 17:01:53  gerd
 * 	Removed the read lock (now do_protected already gets a read
 * lock).
 *
 * Revision 1.12  2000/04/10 21:46:04  gerd
 * 	Beautifications.
 *
 * Revision 1.11  2000/04/09 23:47:16  gerd
 * 	Layout changes.
 *
 * Revision 1.10  2000/04/09 17:13:03  gerd
 * 	Update: renaming Buffer into Wbuffer.
 *
 * Revision 1.9  1999/05/20 21:49:16  gerd
 * 	Design improvements.
 *
 * Revision 1.8  1999/05/19 19:48:30  gerd
 * 	Changes colors in generated HTML.
 *
 * Revision 1.7  1999/05/18 20:27:05  gerd
 * 	Added sections in the generated output.
 * 	Added "sort by classification".
 * 	Revised the design and the printed fields.
 *
 * Revision 1.6  1999/05/13 12:50:32  gerd
 * 	Minimal changes because of updates of other files.
 *
 * Revision 1.5  1999/04/25 18:32:59  gerd
 * 	Minor changes.
 *
 * Revision 1.4  1999/04/18 23:08:27  gerd
 * 	Major change: The CGI programs can now handle with typed DB tables.
 * 	Change: link_change.fragment no longer exists
 *
 * Revision 1.3  1999/03/14 01:08:39  gerd
 * 	Added: modification date as new record component
 * 	Added: by default, items are listed by modification date
 * 	Changed: passwords are stored as MD5 message digest
 * 	Added: can now change existing records field by field
 * 	Improved: design of the web pages
 *
 * Revision 1.2  1999/02/21 17:52:14  gerd
 * 	Added automatic parameters.
 * 	Added support for browsers without frames.
 *
 * Revision 1.1  1999/02/18 00:58:59  gerd
 * 	First version, has been tested to some extend.
 *
 * 
 *)
