(* $Id: cgi.ml,v 1.2 1999/03/18 19:35:28 gerd Exp $
 * ----------------------------------------------------------------------
 * This file is a copy from ocamlcgi release 0.3, see 
 *   http://www.lri.fr/~filliatr/ftp/ocaml/cgi/
 *)

(*
 * ocamlcgi - Objective Caml library for writing CGIs
 * Copyright (C) 1997 Daniel de Rauglaudre, INRIA
 * Copyright (C) 1998 Jean-Christophe FILLIATRE
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2, as published by the Free Software Foundation.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * See the GNU Library General Public License version 2 for more details
 * (enclosed in the file LGPL).
 *)

(*****************************************************)
(* VERSION IN ORIGINAL REPOSITORY                    *)
(* Id: cgi.ml,v 1.5 1999/03/18 16:08:14 filliatr Exp *)
(*****************************************************)

(* Code from wserver.ml, (C) 1997 Daniel de Rauglaudre, INRIA. *)

let hexa_digit x =
  if x >= 10 then Char.chr (Char.code 'A' + x - 10)
  else Char.chr (Char.code '0' + x)
;;

let hexa_val conf =
  match conf with
    '0'..'9' -> Char.code conf - Char.code '0'
  | 'a'..'f' -> Char.code conf - Char.code 'a' + 10
  | 'A'..'F' -> Char.code conf - Char.code 'A' + 10
  | _ -> 0
;;

let decode s =
  let rec need_decode i =
    if i < String.length s then
      match s.[i] with
        '%' | '+' -> true
      | _ -> need_decode (succ i)
    else false
  in
  let rec compute_len i i1 =
    if i < String.length s then
      let i =
        match s.[i] with
          '%' when i + 2 < String.length s -> i + 3
        | _ -> succ i
      in
      compute_len i (succ i1)
    else i1
  in
  let rec copy_decode_in s1 i i1 =
    if i < String.length s then
      let i =
        match s.[i] with
          '%' when i + 2 < String.length s ->
            let v = hexa_val s.[i + 1] * 16 + hexa_val s.[i + 2] in
            s1.[i1] <- Char.chr v; i + 3
        | '+' -> s1.[i1] <- ' '; succ i
        | x -> s1.[i1] <- x; succ i
      in
      copy_decode_in s1 i (succ i1)
    else s1
  in
  let rec strip_heading_and_trailing_spaces s =
    if String.length s > 0 then
      if s.[0] == ' ' then
        strip_heading_and_trailing_spaces
          (String.sub s 1 (String.length s - 1))
      else if s.[String.length s - 1] == ' ' then
        strip_heading_and_trailing_spaces
          (String.sub s 0 (String.length s - 1))
      else s
    else s
  in
  if need_decode 0 then
    let len = compute_len 0 0 in
    let s1 = String.create len in
    strip_heading_and_trailing_spaces (copy_decode_in s1 0 0)
  else s
;;

(* special characters must be encoded. According to RFC 1738 they are : *)

let special = function 
  | '\000'..'\031' | '\127'..'\255'                      (* non US ASCII *)
  | ' ' | '<' | '>' | '"' | '#' | '%' 
  | '{' | '}' | '|' | '\\' | '^' | '~' | '[' | ']' | '`' (* unsafe *)
  | ';' | '/' | '?' | ':' | '@' | '=' | '&'              (* reserved *)
      -> true
  | _ -> false

let encode s =
  let rec need_code i =
    if i < String.length s then
      match s.[i] with
        ' ' -> true
      | x -> if special x then true else need_code (succ i)
    else false
  in
  let rec compute_len i i1 =
    if i < String.length s then
      let i1 = if special s.[i] then i1 + 3 else succ i1 in
      compute_len (succ i) i1
    else i1
  in
  let rec copy_code_in s1 i i1 =
    if i < String.length s then
      let i1 =
        match s.[i] with
          ' ' -> s1.[i1] <- '+'; succ i1
        | c ->
            if special c then
              begin
                s1.[i1] <- '%';
                s1.[i1 + 1] <- hexa_digit (Char.code c / 16);
                s1.[i1 + 2] <- hexa_digit (Char.code c mod 16);
                i1 + 3
              end
            else begin s1.[i1] <- c; succ i1 end
      in
      copy_code_in s1 (succ i) i1
    else s1
  in
  if need_code 0 then
    let len = compute_len 0 0 in copy_code_in (String.create len) 0 0
  else s
;;


(* Using the following function, we avoid the use of the Str library
 * which is not compatible with the threads of ocaml.
 * Thanks to Olivier Montanuy. *)

let split separator text =
  let len = String.length text in
  let rec loop pos =
    if pos < len then
      try
       	let last = String.index_from text pos separator in
 	let str = String.sub text pos (last-pos) in
	  str::(loop (succ last))
      with Not_found ->
 	if pos < len then [String.sub text pos (len-pos)]
 	else []
    else []
  in
  loop 0

(* parse_args: parsing of the CGI arguments *)

let safe_getenv s =
  try Sys.getenv s
  with Not_found -> 
    failwith ("Cgi: the environment variable "^s^" is not set")

let parse_args () = 
  let s = 
    if safe_getenv "REQUEST_METHOD" = "GET" then
      safe_getenv "QUERY_STRING"
    else
      let n = int_of_string (safe_getenv "CONTENT_LENGTH") in
      let buf = String.create n in
      really_input stdin buf 0 n;
      buf
  in
  let assocs = split '&' s in
  let one_assoc s =
    try
      let i = String.index s '=' in
      String.sub s 0 i, 
      decode (String.sub s (succ i) (String.length s - i - 1))
    with
	Not_found -> s,""
  in
  List.map one_assoc assocs


(* content-type *)

let header m =
  Printf.printf "Content-type: %s\n\n" (if m="" then "text/html" else m)


(* returns the URL of the CGI *)

let this_url () =
  "http://" ^ (safe_getenv "SERVER_NAME") ^ (safe_getenv "SCRIPT_NAME")

(* ======================================================================
 * History:
 * 
 * $Log: cgi.ml,v $
 * Revision 1.2  1999/03/18 19:35:28  gerd
 * 	Replaced by newest release 0.3 from www.lri.fr
 *
 * Revision 1.1  1999/02/18 00:59:01  gerd
 * 	First version, has been tested to some extend.
 *
 * 
 *)

