(* $Id: layout.ml,v 1.7 1999/05/20 21:46:05 gerd Exp $
 * ----------------------------------------------------------------------
 *
 *)

open Field
open Common
open Scheme
open Fragment

class t the_scheme =
  object (self)

    val scheme = the_scheme

    method html_display fragment r =
      include_file
	fragment
	[]
	(parameters_of_record scheme r);



    method html_input_form_spec (javascript:bool) 
                                (r : (string * Field.field) list) =
      
      let field n =
	try
	  List.assoc n r 
	with
	  Not_found -> Field.null
      in

      let vals =
	List.map
	  (fun (n,t) ->
	    let (w,h) = self # html_input_size n in
	    (* Logging.write ("form " ^ n); *)
	    let n' = "att_" ^ n in
	    n',
	    Verbatim
	      (match get_base_type t with
		T_line       -> html_input_line n' w (field n)
	      | T_password   -> html_input_password n' w (field n)
	      |	T_textbox    -> html_input_textbox n' w h (field n)
	      | T_hyperlink  -> html_input_hyperlink n' javascript w (field n)
	      | T_int        -> html_input_int n' javascript w (field n)
	      | T_date       -> html_input_date n' javascript w (field n)
	      |	T_constrained (_,_) -> failwith "internal error"
	      | T_select sel -> 
		  let rv = field n in
		  let sv =
		    if rv = null then
		      mk_select_default t
		    else
		      rv in
		  html_input_select_asbox n' h sv))
	  (physical_typedef_of_scheme scheme)
      in
      vals



    method html_input_form fragment javascript r =
      include_file
	fragment
	[]
	(self # html_input_form_spec javascript r)


    method html_input_size name =
      (60,5)
  end


(* ======================================================================
 * History:
 * 
 * $Log: layout.ml,v $
 * Revision 1.7  1999/05/20 21:46:05  gerd
 * 	Removed the computation of the hyperlink text.
 *
 * Revision 1.6  1999/05/18 19:54:20  gerd
 * 	Changed the default number of columns of input boxes from
 * 80 to 60.
 *
 * Revision 1.5  1999/05/13 12:18:46  gerd
 * 	Change: In 'html_input_form_spec' only physical fields are converted
 * to input boxes.
 *
 * Revision 1.4  1999/05/03 23:22:15  gerd
 * 	Change: html_input_form does not need a Record.record any longer.
 * It is sufficient to pass a (string * Field.field) list which is allowed
 * to contain unchecked fields.
 *
 * Revision 1.3  1999/04/25 21:36:27  gerd
 * 	Renamed the @name@s of attributes in @att_name@.
 *
 * Revision 1.2  1999/04/25 18:20:30  gerd
 * 	Minor changes.
 *
 * Revision 1.1  1999/04/18 23:14:11  gerd
 * 	Major change: The Db tables have now typed attributes. The
 * attributes are represented as Field.field, grouped in records by
 * Record.record and stored in tables by File_base.t as before. A Scheme.scheme
 * is the signature of a table.
 * 	Change: Layout tries to encapsulate some of the layout properties
 * of tables.
 *
 * 
 *)
