(* $Id: logging.ml,v 1.6 2000/04/10 23:55:37 gerd Exp $
 * ----------------------------------------------------------------------
 *
 *)

(* Basic logging functionality *)

open Str;;

type loglevel =
    Debug | Param | Startup | Info | Warn | Err;;

let logfile = "../ocamllinkdb.db/all.log";;               (* specify the log file *)

let loglevels = ref [ Param; Info; Warn; Err ];;

let log = lazy (open_out_gen [ Open_wronly; Open_append; Open_creat;
			       Open_text ]
		  0o644
		  logfile);;

let last_re = regexp "^.*/\\([^/]+\\)$";;

let script = 
  lazy begin
    let args = Sys.argv in
    let rec search k =
      if args.(k) <> "" & args.(k).[0] <> '-' then
	args.(k)
      else
	search (k+1)
    in
    let with_slashes =
      if !Sys.interactive then 
	try search 1 with _ -> "UNKNOWN"
      else
	args.(0)
    in
    if string_match last_re with_slashes 0 then
      matched_group 1 with_slashes
    else
      with_slashes
  end;;

let months = [| "Jan"; "Feb"; "Mar"; "Apr"; "May"; "Jun";
		"Jul"; "Aug"; "Sep"; "Oct"; "Nov"; "Dec" |];;

let do_logging = true;;

let write level s =
  (* write a message to the log file *)
  if do_logging & List.mem level !loglevels then begin
    let lev = 
      List.assoc
	level
	[ Debug, "Debug"; 
	  Param, "Param";
	  Startup, "Startup";
	  Info, "Info";
	  Warn, "Warn";
	  Err, "Err";
	]
    in
    let t = Unix.gmtime (Unix.time()) in
    let t_str = 
      string_of_int (t.Unix.tm_mday) ^ "-" ^ 
      months.(t.Unix.tm_mon) ^ "-" ^ 
      string_of_int (t.Unix.tm_year + 1900) ^ " " ^
      string_of_int (t.Unix.tm_hour) ^ ":" ^ 
      string_of_int (t.Unix.tm_min) ^ ":" ^
      string_of_int (t.Unix.tm_sec)
    in
    let f = Lazy.force log in
    output_string f (t_str ^ " [" ^ Lazy.force script ^ "] [" ^ lev ^ "] " ^ s ^ "\n");
    flush f
  end
;;

(* ======================================================================
 * History:
 * 
 * $Log: logging.ml,v $
 * Revision 1.6  2000/04/10 23:55:37  gerd
 * 	Changed the location of the database and the log file.
 *
 * Revision 1.5  2000/04/10 19:28:29  gerd
 * 	More lightweight logging.
 *
 * Revision 1.4  1999/03/18 19:39:13  gerd
 * 	Fixed some bugs in the comment structure. The problem is that
 * ocamlc recognizes string literals within comments,  see the comments
 * that are marked with "OCAML SCANNER PROBLEM".
 *
 * Revision 1.3  1999/03/17 00:28:06  gerd
 * 	Added comments. These are a mixture of explanations of the
 * design and of notes about the implementation details. The design
 * comments should later go into mli files.
 *
 * Revision 1.2  1999/03/14 01:08:41  gerd
 * 	Added: modification date as new record component
 * 	Added: by default, items are listed by modification date
 * 	Changed: passwords are stored as MD5 message digest
 * 	Added: can now change existing records field by field
 * 	Improved: design of the web pages
 *
 * Revision 1.1  1999/02/18 00:59:02  gerd
 * 	First version, has been tested to some extend.
 *
 * 
 *)
